"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var utils_1 = require("../../shared/utils");
var setup_1 = require("./setup");
var package_const_1 = require("../../shared/constants/package.const");
var messages_1 = require("../../shared/messages");
var scaffolding_helpers_1 = require("./scaffolding.helpers");
var generate_helpers_1 = require("./generate.helpers");
var templateOptions = Object.keys(setup_1.generateSetup);
exports.generateBinCommand = function (prog) {
    prog
        .command('generate <pkg>')
        .describe('Generate a new package')
        .alias('g')
        .example('generate packageName')
        .option('t, template', "Specify a template.\n     Available templates: [" + templateOptions.join(', ') + "]\n     \n     ")
        .example("generate packageName --template " + templateOptions[0])
        .action(function (pkgName, _a) {
        var template = _a.template;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageName, packageTemplateType, workspaceRoot, packageJsonPath, _b, hostName, workspaces, license, workspacePackages, packageSetupPath, bootSpinner, packageName_1, packageDir, author, templateConfig, packageJsonPreset, err_1, dependencies, preparingSpinner, err_2;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        packageName = pkgName;
                        packageTemplateType = template;
                        return [4 /*yield*/, utils_1.findWorkspaceRootDir()];
                    case 1:
                        workspaceRoot = _c.sent();
                        packageJsonPath = path_1.default.resolve(workspaceRoot, package_const_1.PACKAGE_JSON);
                        return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                    case 2:
                        _b = (_c.sent()), hostName = _b.name, workspaces = _b.workspaces, license = _b.license;
                        workspacePackages = utils_1.getWorkspacePackageDirs(workspaces);
                        packageSetupPath = utils_1.getWorkspacePackageSetupPath(workspacePackages);
                        bootSpinner = ora_1.default(messages_1.generateMessage.generating(pkgName));
                        _c.label = 3;
                    case 3:
                        _c.trys.push([3, 9, , 10]);
                        return [4 /*yield*/, scaffolding_helpers_1.getSafeName({
                                basePath: path_1.default.resolve(workspaceRoot + "/" + packageSetupPath),
                                name: pkgName,
                                onPromptMessage: function (name) {
                                    var message = messages_1.generateMessage.failed(name);
                                    bootSpinner.fail(messages_1.generateMessage.failed(name));
                                    return message;
                                },
                                onPromptInitial: function (name) { return messages_1.generateMessage.copy(name); }
                            })];
                    case 4:
                        packageName_1 = _c.sent();
                        packageDir = path_1.default.resolve(workspaceRoot, packageSetupPath, packageName_1);
                        return [4 /*yield*/, generate_helpers_1.getPackageTemplateType(template, function () {
                                bootSpinner.fail(messages_1.generateMessage.invalidTemplate(template));
                            })];
                    case 5:
                        packageTemplateType = _c.sent();
                        bootSpinner.start();
                        return [4 /*yield*/, scaffolding_helpers_1.copyTemplate({
                                dir: packageDir,
                                bin: 'generate',
                                template: packageTemplateType
                            })];
                    case 6:
                        _c.sent();
                        bootSpinner.stop();
                        return [4 /*yield*/, scaffolding_helpers_1.getAuthor()];
                    case 7:
                        author = _c.sent();
                        bootSpinner.start();
                        scaffolding_helpers_1.setAuthorName(author);
                        process.chdir(packageDir);
                        templateConfig = setup_1.generateSetup[packageTemplateType];
                        packageJsonPreset = generate_helpers_1.composePackageJson({
                            author: author,
                            name: packageName_1,
                            license: license,
                            hostName: hostName,
                            template: templateConfig
                        });
                        return [4 /*yield*/, scaffolding_helpers_1.createPackageJson({ dir: packageDir, preset: packageJsonPreset })];
                    case 8:
                        _c.sent();
                        bootSpinner.succeed(messages_1.generateMessage.successful(packageName_1));
                        return [3 /*break*/, 10];
                    case 9:
                        err_1 = _c.sent();
                        bootSpinner.fail(messages_1.generateMessage.failed(packageName));
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 10];
                    case 10:
                        dependencies = setup_1.generateSetup[packageTemplateType].dependencies;
                        preparingSpinner = ora_1.default(messages_1.generateMessage.preparingPackage(packageName, dependencies.sort())).start();
                        _c.label = 11;
                    case 11:
                        _c.trys.push([11, 14, , 15]);
                        return [4 /*yield*/, scaffolding_helpers_1.sortPackageJson()];
                    case 12:
                        _c.sent();
                        return [4 /*yield*/, generate_helpers_1.buildPackage()];
                    case 13:
                        _c.sent();
                        preparingSpinner.succeed(messages_1.generateMessage.successfulConfigure());
                        console.log(messages_1.generateMessage.preparedPackage(packageName));
                        return [3 /*break*/, 15];
                    case 14:
                        err_2 = _c.sent();
                        preparingSpinner.fail(messages_1.generateMessage.failedConfigure());
                        utils_1.logError(err_2);
                        process.exit(1);
                        return [3 /*break*/, 15];
                    case 15: return [2 /*return*/];
                }
            });
        });
    });
};
