"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildPackage = exports.composePackageJson = exports.getPackageTemplateType = exports.chooseTemplatePrompt = exports.safePackageName = void 0;
var tslib_1 = require("tslib");
var enquirer_1 = require("enquirer");
var utils_1 = require("../../shared/utils");
var generate_1 = require("./setup/generate");
var shelljs_1 = require("shelljs");
exports.safePackageName = function (name) {
    return name
        .toLowerCase()
        .replace(/(^@.*\/)|((^[^a-zA-Z]+)|[^\w.-])|([^a-zA-Z0-9]+$)/g, '');
};
exports.chooseTemplatePrompt = new enquirer_1.Select({
    message: 'Choose a template',
    choices: Object.keys(generate_1.generateSetup).map(function (option) { return ({
        name: option,
        message: utils_1.info(option)
    }); })
});
exports.getPackageTemplateType = function (template, onInvalidTemplate) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        if (template !== undefined && generate_1.generateSetup[template] === undefined) {
            onInvalidTemplate();
        }
        if (template === undefined || generate_1.generateSetup[template] === undefined) {
            return [2 /*return*/, exports.chooseTemplatePrompt.run()];
        }
        return [2 /*return*/, template];
    });
}); };
exports.composePackageJson = function (_a) {
    var author = _a.author, name = _a.name, hostName = _a.hostName, license = _a.license, template = _a.template;
    var slashNameIndex = hostName.indexOf('/');
    var namespace = slashNameIndex === -1 ? "@" + hostName : hostName.slice(0, slashNameIndex);
    var safeName = exports.safePackageName(name);
    var packageName = namespace ? namespace + "/" + safeName : safeName;
    var packageJson = tslib_1.__assign(tslib_1.__assign({}, template.packageJson), { name: packageName, author: author });
    if (license) {
        packageJson.license = license;
    }
    return packageJson;
};
exports.buildPackage = function () { return shelljs_1.exec('yarn build', { silent: true }); };
